package sun.security.provider;

import java.util.Locale;
import java.security.Provider;
import java.net.URISyntaxException;
import java.io.File;
import java.security.Security;
import java.net.MalformedURLException;
import java.io.InputStreamReader;
import sun.security.util.PolicyUtil;
import java.util.ArrayList;
import java.util.List;
import java.net.URI;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.security.DomainLoadStoreParameter;
import java.security.cert.CertificateException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collections;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import java.util.Date;
import java.security.cert.Certificate;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import java.util.AbstractMap;
import java.security.KeyStoreException;
import java.util.Collection;
import java.security.Key;
import java.util.HashMap;
import java.security.KeyStore;
import java.util.Map;
import java.security.KeyStoreSpi;

abstract class DomainKeyStore extends KeyStoreSpi
{
    private static final String ENTRY_NAME_SEPARATOR = "entrynameseparator";
    private static final String KEYSTORE_PROVIDER_NAME = "keystoreprovidername";
    private static final String KEYSTORE_TYPE = "keystoretype";
    private static final String KEYSTORE_URI = "keystoreuri";
    private static final String KEYSTORE_PASSWORD_ENV = "keystorepasswordenv";
    private static final String REGEX_META = ".$|()[{^?*+\\";
    private static final String DEFAULT_STREAM_PREFIX = "iostream";
    private int streamCounter;
    private String entryNameSeparator;
    private String entryNameSeparatorRegEx;
    private static final String DEFAULT_KEYSTORE_TYPE;
    private final Map<String, KeyStore> keystores;
    
    DomainKeyStore() {
        this.streamCounter = 1;
        this.entryNameSeparator = " ";
        this.entryNameSeparatorRegEx = " ";
        this.keystores = new HashMap();
    }
    
    abstract String convertAlias(String p0);
    
    @Override
    public Key engineGetKey(String s, char[] password) throws NoSuchAlgorithmException, UnrecoverableKeyException {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        Key key = null;
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                key = ((KeyStore)iterator.next()).getKey(alias, password);
                if (key != null) {
                    break;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return key;
    }
    
    @Override
    public Certificate[] engineGetCertificateChain(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        Certificate[] certificateChain = null;
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                certificateChain = ((KeyStore)iterator.next()).getCertificateChain(alias);
                if (certificateChain != null) {
                    break;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return certificateChain;
    }
    
    @Override
    public Certificate engineGetCertificate(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        Certificate certificate = null;
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                certificate = ((KeyStore)iterator.next()).getCertificate(alias);
                if (certificate != null) {
                    break;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return certificate;
    }
    
    @Override
    public Date engineGetCreationDate(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        Date creationDate = null;
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                creationDate = ((KeyStore)iterator.next()).getCreationDate(alias);
                if (creationDate != null) {
                    break;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return creationDate;
    }
    
    @Override
    public void engineSetKeyEntry(String str, Key key, char[] password, Certificate[] chain) throws KeyStoreException {
        AbstractMap.SimpleEntry<String, AbstractMap.SimpleEntry<String, KeyStore>> keystoreForWriting = this.getKeystoreForWriting(str);
        if (keystoreForWriting == null) {
            throw new KeyStoreException("Error setting key entry for '" + str + "'");
        }
        ((KeyStore)((Map.Entry)keystoreForWriting.getValue()).getValue()).setKeyEntry((String)keystoreForWriting.getKey(), key, password, chain);
        return;
    }
    
    @Override
    public void engineSetKeyEntry(String str, byte[] key, Certificate[] chain) throws KeyStoreException {
        AbstractMap.SimpleEntry<String, AbstractMap.SimpleEntry<String, KeyStore>> keystoreForWriting = this.getKeystoreForWriting(str);
        if (keystoreForWriting == null) {
            throw new KeyStoreException("Error setting protected key entry for '" + str + "'");
        }
        ((KeyStore)((Map.Entry)keystoreForWriting.getValue()).getValue()).setKeyEntry((String)keystoreForWriting.getKey(), key, chain);
        return;
    }
    
    @Override
    public void engineSetCertificateEntry(String str, Certificate cert) throws KeyStoreException {
        AbstractMap.SimpleEntry<String, AbstractMap.SimpleEntry<String, KeyStore>> keystoreForWriting = this.getKeystoreForWriting(str);
        if (keystoreForWriting == null) {
            throw new KeyStoreException("Error setting certificate entry for '" + str + "'");
        }
        ((KeyStore)((Map.Entry)keystoreForWriting.getValue()).getValue()).setCertificateEntry((String)keystoreForWriting.getKey(), cert);
        return;
    }
    
    @Override
    public void engineDeleteEntry(String str) throws KeyStoreException {
        AbstractMap.SimpleEntry<String, AbstractMap.SimpleEntry<String, KeyStore>> keystoreForWriting = this.getKeystoreForWriting(str);
        if (keystoreForWriting == null) {
            throw new KeyStoreException("Error deleting entry for '" + str + "'");
        }
        ((KeyStore)((Map.Entry)keystoreForWriting.getValue()).getValue()).deleteEntry((String)keystoreForWriting.getKey());
        return;
    }
    
    @Override
    public Enumeration<String> engineAliases() {
        return new Enumeration<String>() {
            private int index = 0;
            private Map.Entry<String, KeyStore> keystoresEntry = null;
            private String prefix = null;
            private Enumeration<String> aliases = null;
            
            @Override
            public boolean hasMoreElements() {
                try {
                    if (this.aliases == null) {
                        if (iterator.hasNext()) {
                            this.keystoresEntry = (Map.Entry)iterator.next();
                            this.prefix = (String)this.keystoresEntry.getKey() + DomainKeyStore.this.entryNameSeparator;
                            this.aliases = ((KeyStore)this.keystoresEntry.getValue()).aliases();
                        }
                        else {
                            return false;
                        }
                    }
                    if (this.aliases.hasMoreElements()) {
                        return true;
                    }
                    if (iterator.hasNext()) {
                        this.keystoresEntry = (Map.Entry)iterator.next();
                        this.prefix = (String)this.keystoresEntry.getKey() + DomainKeyStore.this.entryNameSeparator;
                        this.aliases = ((KeyStore)this.keystoresEntry.getValue()).aliases();
                    }
                    else {
                        return false;
                    }
                }
                catch (KeyStoreException ex) {
                    return false;
                }
                return this.aliases.hasMoreElements();
            }
            
            @Override
            public String nextElement() {
                if (this.hasMoreElements()) {
                    return this.prefix + (String)this.aliases.nextElement();
                }
                throw new NoSuchElementException();
            }
        };
    }
    
    @Override
    public boolean engineContainsAlias(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                if (((KeyStore)iterator.next()).containsAlias(alias)) {
                    return true;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return false;
    }
    
    @Override
    public int engineSize() {
        int n = 0;
        try {
            Iterator<KeyStore> iterator = this.keystores.values().iterator();
            while (iterator.hasNext()) {
                n += ((KeyStore)iterator.next()).size();
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return n;
    }
    
    @Override
    public boolean engineIsKeyEntry(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                if (((KeyStore)iterator.next()).isKeyEntry(alias)) {
                    return true;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return false;
    }
    
    @Override
    public boolean engineIsCertificateEntry(String s) {
        AbstractMap.SimpleEntry<String, Collection<KeyStore>> keystoresForReading = this.getKeystoresForReading(s);
        try {
            String alias = (String)keystoresForReading.getKey();
            Iterator<Object> iterator = ((Collection)keystoresForReading.getValue()).iterator();
            while (iterator.hasNext()) {
                if (((KeyStore)iterator.next()).isCertificateEntry(alias)) {
                    return true;
                }
                continue;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        return false;
    }
    
    private AbstractMap.SimpleEntry<String, Collection<KeyStore>> getKeystoresForReading(String key) {
        String[] split = key.split(this.entryNameSeparatorRegEx, 2);
        if (split.length == 2) {
            KeyStore o = (KeyStore)this.keystores.get(split[0]);
            if (o != null) {
                return new AbstractMap.SimpleEntry(split[1], Collections.singleton(o));
            }
        }
        else if (split.length == 1) {
            return new AbstractMap.SimpleEntry(key, this.keystores.values());
        }
        return new AbstractMap.SimpleEntry("", Collections.emptyList());
    }
    
    private AbstractMap.SimpleEntry<String, AbstractMap.SimpleEntry<String, KeyStore>> getKeystoreForWriting(String s) {
        String[] split = s.split(this.entryNameSeparator, 2);
        if (split.length == 2) {
            KeyStore value = (KeyStore)this.keystores.get(split[0]);
            if (value != null) {
                return new AbstractMap.SimpleEntry(split[1], new AbstractMap.SimpleEntry(split[0], value));
            }
        }
        return null;
    }
    
    @Override
    public String engineGetCertificateAlias(Certificate cert) {
        try {
            String certificateAlias = null;
            Iterator<KeyStore> iterator = this.keystores.values().iterator();
            while (iterator.hasNext() && (certificateAlias = ((KeyStore)iterator.next()).getCertificateAlias(cert)) == null) {}
            return certificateAlias;
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
    }
    
    @Override
    public void engineStore(OutputStream stream, char[] password) throws IOException, NoSuchAlgorithmException, CertificateException {
        try {
            if (this.keystores.size() == 1) {
                ((KeyStore)this.keystores.values().iterator().next()).store(stream, password);
                return;
            }
        }
        catch (KeyStoreException cause) {
            throw new IllegalStateException(cause);
        }
        throw new UnsupportedOperationException("This keystore must be stored using a DomainLoadStoreParameter");
    }
    
    @Override
    public void engineStore(KeyStore.LoadStoreParameter loadStoreParameter) throws IOException, NoSuchAlgorithmException, CertificateException {
        if ((loadStoreParameter instanceof DomainLoadStoreParameter)) {
            DomainLoadStoreParameter domainLoadStoreParameter = (DomainLoadStoreParameter)loadStoreParameter;
            for (DomainKeyStore.KeyStoreBuilderComponents keyStoreBuilderComponents : this.getBuilders(domainLoadStoreParameter.getConfiguration(), domainLoadStoreParameter.getProtectionParams())) {
                try {
                    if (!(keyStoreBuilderComponents.protection instanceof KeyStore.PasswordProtection)) {
                        throw new KeyStoreException(new IllegalArgumentException("ProtectionParameter must be a KeyStore.PasswordProtection"));
                    }
                    char[] password = ((KeyStore.PasswordProtection)keyStoreBuilderComponents.protection).getPassword();
                    KeyStore keyStore = (KeyStore)this.keystores.get(keyStoreBuilderComponents.name);
                    try (final Object stream = new FileOutputStream(keyStoreBuilderComponents.file)) {
                        keyStore.store(stream, password);
                        if (stream != null) {
                            Throwable t;
                            if (t != null) {
                                try {
                                    stream.close();
                                }
                                catch (Throwable exception) {
                                    t.addSuppressed(exception);
                                }
                            }
                            else {
                                stream.close();
                            }
                        }
                        else {
                            continue;
                        }
                    }
                }
                catch (KeyStoreException cause) {
                    throw new IOException(cause);
                }
            }
            return;
        }
        throw new UnsupportedOperationException("This keystore must be stored using a DomainLoadStoreParameter");
    }
    
    @Override
    public void engineLoad(InputStream inputStream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        try {
            KeyStore keyStore;
            try {
                keyStore = KeyStore.getInstance("JKS");
                keyStore.load(inputStream, array);
            }
            catch (Exception ex) {
                if (!"JKS".equalsIgnoreCase(DomainKeyStore.DEFAULT_KEYSTORE_TYPE)) {
                    keyStore = KeyStore.getInstance(DomainKeyStore.DEFAULT_KEYSTORE_TYPE);
                    keyStore.load(inputStream, array);
                }
                else {
                    throw ex;
                }
            }
            this.keystores.put("iostream" + this.streamCounter++, keyStore);
        }
        catch (Exception ex2) {
            throw new UnsupportedOperationException("This keystore must be loaded using a DomainLoadStoreParameter");
        }
    }
    
    @Override
    public void engineLoad(KeyStore.LoadStoreParameter loadStoreParameter) throws IOException, NoSuchAlgorithmException, CertificateException {
        if ((loadStoreParameter instanceof DomainLoadStoreParameter)) {
            DomainLoadStoreParameter domainLoadStoreParameter = (DomainLoadStoreParameter)loadStoreParameter;
            for (DomainKeyStore.KeyStoreBuilderComponents keyStoreBuilderComponents : this.getBuilders(domainLoadStoreParameter.getConfiguration(), domainLoadStoreParameter.getProtectionParams())) {
                try {
                    if (keyStoreBuilderComponents.file != null) {
                        this.keystores.put(keyStoreBuilderComponents.name, KeyStore.Builder.newInstance(keyStoreBuilderComponents.type, keyStoreBuilderComponents.provider, keyStoreBuilderComponents.file, keyStoreBuilderComponents.protection).getKeyStore());
                    }
                    else {
                        this.keystores.put(keyStoreBuilderComponents.name, KeyStore.Builder.newInstance(keyStoreBuilderComponents.type, keyStoreBuilderComponents.provider, keyStoreBuilderComponents.protection).getKeyStore());
                    }
                }
                catch (KeyStoreException cause) {
                    throw new IOException(cause);
                }
            }
            return;
        }
        throw new UnsupportedOperationException("This keystore must be loaded using a DomainLoadStoreParameter");
    }
    
    private List<DomainKeyStore.KeyStoreBuilderComponents> getBuilders(URI obj, Map<String, KeyStore.ProtectionParameter> map) throws IOException {
        PolicyParser policyParser = new PolicyParser(true);
        Collection<PolicyParser.DomainEntry> domainEntries = null;
        ArrayList<DomainKeyStore.KeyStoreBuilderComponents> list = new ArrayList();
        String fragment = obj.getFragment();
        try (final InputStreamReader inputStreamReader = new InputStreamReader(PolicyUtil.getInputStream(obj.toURL()), "UTF-8")) {
            policyParser.read(inputStreamReader);
            domainEntries = policyParser.getDomainEntries();
        }
        catch (MalformedURLException cause) {
            throw new IOException(cause);
        }
        catch (PolicyParser.ParsingException cause2) {
            throw new IOException(cause2);
        }
        for (PolicyParser.DomainEntry domainEntry : domainEntries) {
            Map<String, String> properties = domainEntry.getProperties();
            if (fragment != null && !fragment.equalsIgnoreCase(domainEntry.getName())) {
                continue;
            }
            if (properties.containsKey("entrynameseparator")) {
                this.entryNameSeparator = (String)properties.get("entrynameseparator");
                StringBuilder o = new StringBuilder();
                for (int i = 0; i < this.entryNameSeparator.length(); ++i) {
                    int char1 = this.entryNameSeparator.charAt(i);
                    if (".$|()[{^?*+\\".indexOf(char1) != -1) {
                        o.append('\\');
                    }
                    o.append((char)char1);
                }
                this.entryNameSeparatorRegEx = o.toString();
            }
            for (PolicyParser.KeyStoreEntry keyStoreEntry : domainEntry.getEntries()) {
                String name = keyStoreEntry.getName();
                HashMap o2 = new HashMap(properties);
                o2.putAll(keyStoreEntry.getProperties());
                String default_KEYSTORE_TYPE = DomainKeyStore.DEFAULT_KEYSTORE_TYPE;
                if (o2.containsKey("keystoretype")) {
                    default_KEYSTORE_TYPE = (String)o2.get("keystoretype");
                }
                Provider provider = null;
                if (o2.containsKey("keystoreprovidername")) {
                    String s = (String)o2.get("keystoreprovidername");
                    provider = Security.getProvider(s);
                    if (provider == null) {
                        throw new IOException("Error locating JCE provider: " + s);
                    }
                }
                Object o3 = null;
                if (o2.containsKey("keystoreuri")) {
                    String str = (String)o2.get("keystoreuri");
                    try {
                        if (str.startsWith("file://")) {
                            o3 = new File(new URI(str));
                        }
                        else {
                            o3 = new File(str);
                        }
                    }
                    catch (URISyntaxException | IllegalArgumentException cause3) {
                        throw new IOException("Error processing keystore property: keystoreURI=\"" + str + "\"", cause3);
                    }
                }
                KeyStore.ProtectionParameter protectionParameter;
                if (map.containsKey(name)) {
                    protectionParameter = (Object)map.get(name);
                }
                else if (o2.containsKey("keystorepasswordenv")) {
                    String s2 = (String)o2.get("keystorepasswordenv");
                    String getenv = System.getenv(s2);
                    if (getenv != null) {
                        protectionParameter = new KeyStore.PasswordProtection(getenv.toCharArray());
                    }
                    else {
                        throw new IOException("Error processing keystore property: keystorePasswordEnv=\"" + s2 + "\"");
                    }
                }
                else {
                    protectionParameter = new KeyStore.PasswordProtection(null);
                }
                list.add(new DomainKeyStore.KeyStoreBuilderComponents(name, default_KEYSTORE_TYPE, provider, o3, protectionParameter));
            }
        }
        if (list.isEmpty()) {
            throw new IOException("Error locating domain configuration data for: " + obj);
        }
        return list;
    }
    
    static {
        DEFAULT_KEYSTORE_TYPE = KeyStore.getDefaultType();
    }
    
    public static final class DKS extends DomainKeyStore
    {
        @Override
        String convertAlias(String s) {
            return s.toLowerCase(Locale.ENGLISH);
        }
    }
    
    class KeyStoreBuilderComponents
    {
        String name;
        String type;
        Provider provider;
        File file;
        KeyStore.ProtectionParameter protection;
        
        KeyStoreBuilderComponents(String name, String type, Provider provider, File file, KeyStore.ProtectionParameter protection) {
            this.name = name;
            this.type = type;
            this.provider = provider;
            this.file = file;
            this.protection = protection;
        }
    }
}
